#include "joystick_objects.h"
#include "joystick_resources.h"
#include "joystick_main.h"
#include "built_in.h"


// TFT module connections
char TFT_DataPort at LATE;
sbit TFT_RST at LATC1_bit;
sbit TFT_BLED at LATA9_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATF12_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_BLED_Direction at TRISA9_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISF12_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at LATB13_bit;
sbit DriveX_Right at LATB11_bit;
sbit DriveY_Up at LATB12_bit;
sbit DriveY_Down at LATB10_bit;
sbit DriveX_Left_Direction at TRISB13_bit;
sbit DriveX_Right_Direction at TRISB11_bit;
sbit DriveY_Up_Direction at TRISB12_bit;
sbit DriveY_Down_Direction at TRISB10_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 1000;
char PenDown;
typedef unsigned long TPointer;
TPointer PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TCButton *local_cbutton;
TCButton *exec_cbutton;
int cbutton_order;
TBox *local_box;
TBox *exec_box;
int box_order;

void PMPWaitBusy() {
  while(PMMODEbits.BUSY);
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  PMDIN = index;
  PMPWaitBusy();
}

void Write_Command( unsigned short cmd ) {
  TFT_RS = 1;
  PMDIN = cmd;
  PMPWaitBusy();
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  PMDIN = _data;
  PMPWaitBusy();
}


void Init_ADC() {
  AD1PCFG = 0xFFFF;
  PCFG12_bit = 0;
  PCFG13_bit = 0;
  // PMP setup
  ADC1_Init();
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init(320, 240);

  TP_TFT_Init(320, 240, 13, 12);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TCButton                Button1 = 
         {
         &Screen1              , //   Button1.OwnerScreen
         0                     , //   Button1.Order           
         202                   , //   Button1.Left            
         55                    , //   Button1.Top              
         50                    , //   Button1.Width            
         25                    , //   Button1.Height        
         1                     , //   Button1.Pen_Width          
         0x0000                , //   Button1.Pen_Color       
         1                     , //   Button1.Visible         
         1                     , //   Button1.Active           
         1                     , //   Button1.Transparent     
         &Button1_Caption      , //   Button1.Caption        
         _taCenter             , //   Button1.TextAlign        
         &Tahoma11x13_Regular  , //   Button1.FontName        
         0x0000                , //
         1                     , //   Button1.Gradient        
         0                     , //   Button1.Gradient_Orientation    
         0xFFFF                , //   Button1.Gradient_Start_Color    
         0xC618                , //  Button1.Gradient_Start_Color    
         0xC618                , //   Button1.Color           
         1                     , //   Button1.PressColEnabled 
         0xE71C                , //   Button1.Press_Color     
         Button1OnUp           , //   Button1.OnUpPtr
         Button1OnDown         , //   Button1.OnDownPtr
         0                     , //   Button1.OnClickPtr
         0                       //   Button1.OnPressPtr
         };
const char Button1_Caption[2] = "1";

  TCButton                Button2 = 
         {
         &Screen1              , //   Button2.OwnerScreen
         1                     , //   Button2.Order           
         202                   , //   Button2.Left            
         80                    , //   Button2.Top              
         50                    , //   Button2.Width            
         25                    , //   Button2.Height        
         1                     , //   Button2.Pen_Width          
         0x0000                , //   Button2.Pen_Color       
         1                     , //   Button2.Visible         
         1                     , //   Button2.Active           
         1                     , //   Button2.Transparent     
         &Button2_Caption      , //   Button2.Caption        
         _taCenter             , //   Button2.TextAlign        
         &Tahoma11x13_Regular  , //   Button2.FontName        
         0x0000                , //
         1                     , //   Button2.Gradient        
         0                     , //   Button2.Gradient_Orientation    
         0xFFFF                , //   Button2.Gradient_Start_Color    
         0xC618                , //  Button2.Gradient_Start_Color    
         0xC618                , //   Button2.Color           
         1                     , //   Button2.PressColEnabled 
         0xE71C                , //   Button2.Press_Color     
         Button2OnUp           , //   Button2.OnUpPtr
         Button2OnDown         , //   Button2.OnDownPtr
         0                     , //   Button2.OnClickPtr
         0                       //   Button2.OnPressPtr
         };
const char Button2_Caption[2] = "2";

  TCButton                Button3 = 
         {
         &Screen1              , //   Button3.OwnerScreen
         2                     , //   Button3.Order           
         202                   , //   Button3.Left            
         105                   , //   Button3.Top              
         50                    , //   Button3.Width            
         25                    , //   Button3.Height        
         1                     , //   Button3.Pen_Width          
         0x0000                , //   Button3.Pen_Color       
         1                     , //   Button3.Visible         
         1                     , //   Button3.Active           
         1                     , //   Button3.Transparent     
         &Button3_Caption      , //   Button3.Caption        
         _taCenter             , //   Button3.TextAlign        
         &Tahoma11x13_Regular  , //   Button3.FontName        
         0x0000                , //
         1                     , //   Button3.Gradient        
         0                     , //   Button3.Gradient_Orientation    
         0xFFFF                , //   Button3.Gradient_Start_Color    
         0xC618                , //  Button3.Gradient_Start_Color    
         0xC618                , //   Button3.Color           
         1                     , //   Button3.PressColEnabled 
         0xE71C                , //   Button3.Press_Color     
         Button3OnUp           , //   Button3.OnUpPtr
         Button3OnDown         , //   Button3.OnDownPtr
         0                     , //   Button3.OnClickPtr
         0                       //   Button3.OnPressPtr
         };
const char Button3_Caption[2] = "3";

  TCButton                Button4 = 
         {
         &Screen1              , //   Button4.OwnerScreen
         3                     , //   Button4.Order           
         202                   , //   Button4.Left            
         130                   , //   Button4.Top              
         50                    , //   Button4.Width            
         25                    , //   Button4.Height        
         1                     , //   Button4.Pen_Width          
         0x0000                , //   Button4.Pen_Color       
         1                     , //   Button4.Visible         
         1                     , //   Button4.Active           
         1                     , //   Button4.Transparent     
         &Button4_Caption      , //   Button4.Caption        
         _taCenter             , //   Button4.TextAlign        
         &Tahoma11x13_Regular  , //   Button4.FontName        
         0x0000                , //
         1                     , //   Button4.Gradient        
         0                     , //   Button4.Gradient_Orientation    
         0xFFFF                , //   Button4.Gradient_Start_Color    
         0xC618                , //  Button4.Gradient_Start_Color    
         0xC618                , //   Button4.Color           
         1                     , //   Button4.PressColEnabled 
         0xE71C                , //   Button4.Press_Color     
         Button4OnUp           , //   Button4.OnUpPtr
         Button4OnDown         , //   Button4.OnDownPtr
         0                     , //   Button4.OnClickPtr
         0                       //   Button4.OnPressPtr
         };
const char Button4_Caption[2] = "4";

  TCButton                Button5 = 
         {
         &Screen1              , //   Button5.OwnerScreen
         4                     , //   Button5.Order           
         202                   , //   Button5.Left            
         155                   , //   Button5.Top              
         50                    , //   Button5.Width            
         25                    , //   Button5.Height        
         1                     , //   Button5.Pen_Width          
         0x0000                , //   Button5.Pen_Color       
         1                     , //   Button5.Visible         
         1                     , //   Button5.Active           
         1                     , //   Button5.Transparent     
         &Button5_Caption      , //   Button5.Caption        
         _taCenter             , //   Button5.TextAlign        
         &Tahoma11x13_Regular  , //   Button5.FontName        
         0x0000                , //
         1                     , //   Button5.Gradient        
         0                     , //   Button5.Gradient_Orientation    
         0xFFFF                , //   Button5.Gradient_Start_Color    
         0xC618                , //  Button5.Gradient_Start_Color    
         0xC618                , //   Button5.Color           
         1                     , //   Button5.PressColEnabled 
         0xE71C                , //   Button5.Press_Color     
         Button5OnUp           , //   Button5.OnUpPtr
         Button5OnDown         , //   Button5.OnDownPtr
         0                     , //   Button5.OnClickPtr
         0                       //   Button5.OnPressPtr
         };
const char Button5_Caption[2] = "5";

  TCButton                Button6 = 
         {
         &Screen1              , //   Button6.OwnerScreen
         5                     , //   Button6.Order           
         252                   , //   Button6.Left            
         55                    , //   Button6.Top              
         50                    , //   Button6.Width            
         25                    , //   Button6.Height        
         1                     , //   Button6.Pen_Width          
         0x0000                , //   Button6.Pen_Color       
         1                     , //   Button6.Visible         
         1                     , //   Button6.Active           
         1                     , //   Button6.Transparent     
         &Button6_Caption      , //   Button6.Caption        
         _taCenter             , //   Button6.TextAlign        
         &Tahoma11x13_Regular  , //   Button6.FontName        
         0x0000                , //
         1                     , //   Button6.Gradient        
         0                     , //   Button6.Gradient_Orientation    
         0xFFFF                , //   Button6.Gradient_Start_Color    
         0xC618                , //  Button6.Gradient_Start_Color    
         0xC618                , //   Button6.Color           
         1                     , //   Button6.PressColEnabled 
         0xE71C                , //   Button6.Press_Color     
         Button6OnUp           , //   Button6.OnUpPtr
         Button6OnDown         , //   Button6.OnDownPtr
         0                     , //   Button6.OnClickPtr
         0                       //   Button6.OnPressPtr
         };
const char Button6_Caption[2] = "6";

  TCButton                Button7 = 
         {
         &Screen1              , //   Button7.OwnerScreen
         6                     , //   Button7.Order           
         252                   , //   Button7.Left            
         80                    , //   Button7.Top              
         50                    , //   Button7.Width            
         25                    , //   Button7.Height        
         1                     , //   Button7.Pen_Width          
         0x0000                , //   Button7.Pen_Color       
         1                     , //   Button7.Visible         
         1                     , //   Button7.Active           
         1                     , //   Button7.Transparent     
         &Button7_Caption      , //   Button7.Caption        
         _taCenter             , //   Button7.TextAlign        
         &Tahoma11x13_Regular  , //   Button7.FontName        
         0x0000                , //
         1                     , //   Button7.Gradient        
         0                     , //   Button7.Gradient_Orientation    
         0xFFFF                , //   Button7.Gradient_Start_Color    
         0xC618                , //  Button7.Gradient_Start_Color    
         0xC618                , //   Button7.Color           
         1                     , //   Button7.PressColEnabled 
         0xE71C                , //   Button7.Press_Color     
         Button7OnUp           , //   Button7.OnUpPtr
         Button7OnDown         , //   Button7.OnDownPtr
         0                     , //   Button7.OnClickPtr
         0                       //   Button7.OnPressPtr
         };
const char Button7_Caption[2] = "7";

  TCButton                Button8 = 
         {
         &Screen1              , //   Button8.OwnerScreen
         7                     , //   Button8.Order           
         252                   , //   Button8.Left            
         105                   , //   Button8.Top              
         50                    , //   Button8.Width            
         25                    , //   Button8.Height        
         1                     , //   Button8.Pen_Width          
         0x0000                , //   Button8.Pen_Color       
         1                     , //   Button8.Visible         
         1                     , //   Button8.Active           
         1                     , //   Button8.Transparent     
         &Button8_Caption      , //   Button8.Caption        
         _taCenter             , //   Button8.TextAlign        
         &Tahoma11x13_Regular  , //   Button8.FontName        
         0x0000                , //
         1                     , //   Button8.Gradient        
         0                     , //   Button8.Gradient_Orientation    
         0xFFFF                , //   Button8.Gradient_Start_Color    
         0xC618                , //  Button8.Gradient_Start_Color    
         0xC618                , //   Button8.Color           
         1                     , //   Button8.PressColEnabled 
         0xE71C                , //   Button8.Press_Color     
         Button8OnUp           , //   Button8.OnUpPtr
         Button8OnDown         , //   Button8.OnDownPtr
         0                     , //   Button8.OnClickPtr
         0                       //   Button8.OnPressPtr
         };
const char Button8_Caption[2] = "8";

  TCButton                Button9 = 
         {
         &Screen1              , //   Button9.OwnerScreen
         8                     , //   Button9.Order           
         252                   , //   Button9.Left            
         130                   , //   Button9.Top              
         50                    , //   Button9.Width            
         25                    , //   Button9.Height        
         1                     , //   Button9.Pen_Width          
         0x0000                , //   Button9.Pen_Color       
         1                     , //   Button9.Visible         
         1                     , //   Button9.Active           
         1                     , //   Button9.Transparent     
         &Button9_Caption      , //   Button9.Caption        
         _taCenter             , //   Button9.TextAlign        
         &Tahoma11x13_Regular  , //   Button9.FontName        
         0x0000                , //
         1                     , //   Button9.Gradient        
         0                     , //   Button9.Gradient_Orientation    
         0xFFFF                , //   Button9.Gradient_Start_Color    
         0xC618                , //  Button9.Gradient_Start_Color    
         0xC618                , //   Button9.Color           
         1                     , //   Button9.PressColEnabled 
         0xE71C                , //   Button9.Press_Color     
         Button9OnUp           , //   Button9.OnUpPtr
         Button9OnDown         , //   Button9.OnDownPtr
         0                     , //   Button9.OnClickPtr
         0                       //   Button9.OnPressPtr
         };
const char Button9_Caption[2] = "9";

  TCButton                Button10 = 
         {
         &Screen1              , //   Button10.OwnerScreen
         9                     , //   Button10.Order           
         252                   , //   Button10.Left            
         155                   , //   Button10.Top              
         50                    , //   Button10.Width            
         25                    , //   Button10.Height        
         1                     , //   Button10.Pen_Width          
         0x0000                , //   Button10.Pen_Color       
         1                     , //   Button10.Visible         
         1                     , //   Button10.Active           
         1                     , //   Button10.Transparent     
         &Button10_Caption     , //   Button10.Caption        
         _taCenter             , //   Button10.TextAlign        
         &Tahoma11x13_Regular  , //   Button10.FontName        
         0x0000                , //
         1                     , //   Button10.Gradient        
         0                     , //   Button10.Gradient_Orientation    
         0xFFFF                , //   Button10.Gradient_Start_Color    
         0xC618                , //  Button10.Gradient_Start_Color    
         0xC618                , //   Button10.Color           
         1                     , //   Button10.PressColEnabled 
         0xE71C                , //   Button10.Press_Color     
         Button10OnUp          , //   Button10.OnUpPtr
         Button10OnDown        , //   Button10.OnDownPtr
         0                     , //   Button10.OnClickPtr
         0                       //   Button10.OnPressPtr
         };
const char Button10_Caption[3] = "10";

  TCButton                ButtonUp =
         {
         &Screen1              , //   Button10.OwnerScreen
         11                     , //   Button10.Order
         145                   , //   Button10.Left
         182                   , //   Button10.Top
         30                    , //   Button10.Width
         25                    , //   Button10.Height
         1                     , //   Button10.Pen_Width
         0x0000                , //   Button10.Pen_Color
         1                     , //   Button10.Visible
         1                     , //   Button10.Active
         1                     , //   Button10.Transparent
         &ButtonUp_Caption     , //   Button10.Caption
         _taCenter             , //   Button10.TextAlign
         &Tahoma11x13_Regular  , //   Button10.FontName
         0x0000                , //
         1                     , //   Button10.Gradient
         0                     , //   Button10.Gradient_Orientation
         0xFFFF                , //   Button10.Gradient_Start_Color
         0xC618                , //  Button10.Gradient_Start_Color
         0xC618                , //   Button10.Color
         1                     , //   Button10.PressColEnabled
         0xE71C                , //   Button10.Press_Color
         ButtonUpOnUp          , //   Button10.OnUpPtr
         ButtonUpOnDown        , //   Button10.OnDownPtr
         0                     , //   Button10.OnClickPtr
         0                       //   Button10.OnPressPtr
         };
const char ButtonUp_Caption[3] = "Up";

  TCButton                ButtonDown =
         {
         &Screen1              , //   Button10.OwnerScreen
         12                     , //   Button10.Order
         145                   , //   Button10.Left
         212                   , //   Button10.Top
         30                    , //   Button10.Width
         25                    , //   Button10.Height
         1                     , //   Button10.Pen_Width
         0x0000                , //   Button10.Pen_Color
         1                     , //   Button10.Visible
         1                     , //   Button10.Active
         1                     , //   Button10.Transparent
         &ButtonDown_Caption     , //   Button10.Caption
         _taCenter             , //   Button10.TextAlign
         &Tahoma11x13_Regular  , //   Button10.FontName
         0x0000                , //
         1                     , //   Button10.Gradient
         0                     , //   Button10.Gradient_Orientation
         0xFFFF                , //   Button10.Gradient_Start_Color
         0xC618                , //  Button10.Gradient_Start_Color
         0xC618                , //   Button10.Color
         1                     , //   Button10.PressColEnabled
         0xE71C                , //   Button10.Press_Color
         ButtonDownOnUp          , //   Button10.OnUpPtr
         ButtonDownOnDown        , //   Button10.OnDownPtr
         0                     , //   Button10.OnClickPtr
         0                       //   Button10.OnPressPtr
         };
const char ButtonDown_Caption[5] = "Down";

  TCButton                ButtonLeft =
         {
         &Screen1              , //   Button10.OwnerScreen
         13                     , //   Button10.Order
         110                   , //   Button10.Left
         200                   , //   Button10.Top
         30                    , //   Button10.Width
         25                    , //   Button10.Height
         1                     , //   Button10.Pen_Width
         0x0000                , //   Button10.Pen_Color
         1                     , //   Button10.Visible
         1                     , //   Button10.Active
         1                     , //   Button10.Transparent
         &ButtonLeft_Caption     , //   Button10.Caption
         _taCenter             , //   Button10.TextAlign
         &Tahoma11x13_Regular  , //   Button10.FontName
         0x0000                , //
         1                     , //   Button10.Gradient
         0                     , //   Button10.Gradient_Orientation
         0xFFFF                , //   Button10.Gradient_Start_Color
         0xC618                , //  Button10.Gradient_Start_Color
         0xC618                , //   Button10.Color
         1                     , //   Button10.PressColEnabled
         0xE71C                , //   Button10.Press_Color
         ButtonLeftOnUp          , //   Button10.OnUpPtr
         ButtonLeftOnDown        , //   Button10.OnDownPtr
         0                     , //   Button10.OnClickPtr
         0                       //   Button10.OnPressPtr
         };
const char ButtonLeft_Caption[5] = "Left";

  TCButton                ButtonRight =
         {
         &Screen1              , //   Button10.OwnerScreen
         14                     , //   Button10.Order
         180                   , //   Button10.Left
         200                   , //   Button10.Top
         30                    , //   Button10.Width
         25                    , //   Button10.Height
         1                     , //   Button10.Pen_Width
         0x0000                , //   Button10.Pen_Color
         1                     , //   Button10.Visible
         1                     , //   Button10.Active
         1                     , //   Button10.Transparent
         &ButtonRight_Caption     , //   Button10.Caption
         _taCenter             , //   Button10.TextAlign
         &Tahoma11x13_Regular  , //   Button10.FontName
         0x0000                , //
         1                     , //   Button10.Gradient
         0                     , //   Button10.Gradient_Orientation
         0xFFFF                , //   Button10.Gradient_Start_Color
         0xC618                , //  Button10.Gradient_Start_Color
         0xC618                , //   Button10.Color
         1                     , //   Button10.PressColEnabled
         0xE71C                , //   Button10.Press_Color
         ButtonRightOnUp          , //   Button10.OnUpPtr
         ButtonRightOnDown        , //   Button10.OnDownPtr
         0                     , //   Button10.OnClickPtr
         0                       //   Button10.OnPressPtr
         };
const char ButtonRight_Caption[6] = "Right";

  TBox                   Box1;
  TCButton               * const code Screen1_CButtons[14]=
         {
         &Button1,             
         &Button2,             
         &Button3,             
         &Button4,             
         &Button5,             
         &Button6,             
         &Button7,             
         &Button8,             
         &Button9,             
         &Button10,
         &ButtonUp,
         &ButtonDown,
         &ButtonLeft,
         &ButtonRight
         };
  TBox                   * const code Screen1_Boxes[1]=
         {
         &Box1                 
         };



static void InitializeObjects() {
  Screen1.Color                     = 0xFFFF;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.CButtonsCount             = 14;
  Screen1.CButtons                  = Screen1_CButtons;
  Screen1.BoxesCount                = 1;
  Screen1.Boxes                     = Screen1_Boxes;
  Screen1.ObjectsCount              = 15;


  Box1.OwnerScreen     = &Screen1;
  Box1.Order           = 10;
  Box1.Left            = 32;
  Box1.Top             = 51;
  Box1.Width           = 128;
  Box1.Height          = 128;
  Box1.Pen_Width       = 1;
  Box1.Pen_Color       = 0x0000;
  Box1.Visible         = 1;
  Box1.Active          = 1;
  Box1.Transparent     = 1;
  Box1.Gradient        = 0;
  Box1.Gradient_Orientation = 0;
  Box1.Gradient_Start_Color = 0xFFFF;
  Box1.Gradient_End_Color = 0xC618;
  Box1.Color           = 0xD69A;
  Box1.PressColEnabled = 1;
  Box1.Press_Color     = 0xE71C;
  Box1.OnUpPtr         = 0;
  Box1.OnDownPtr       = 0;
  Box1.OnClickPtr      = 0;
  Box1.OnPressPtr      = Box1OnPress;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetCButton(index)             CurrentScreen->CButtons[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]


void DrawCButton(TCButton *Abutton) {
  if (Abutton->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(Abutton->Transparent, Abutton->Press_Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_End_Color, Abutton->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(Abutton->Transparent, Abutton->Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_Start_Color, Abutton->Gradient_End_Color);
    }
    TFT_Set_Pen(Abutton->Pen_Color, Abutton->Pen_Width);
    TFT_Rectangle(Abutton->Left, Abutton->Top, Abutton->Left + Abutton->Width - 1, Abutton->Top + Abutton->Height - 1);
    TFT_Set_Font(Abutton->FontName, Abutton->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    if (Abutton->TextAlign == _taLeft)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + 4, (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taCenter)
      TFT_Write_Text(Abutton->Caption, (Abutton->Left + (Abutton->Width - caption_length) / 2), (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taRight)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + (Abutton->Width - caption_length - 4), (Abutton->Top + (Abutton->Height - caption_height) / 2));
  }
}

void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(ABox->Transparent, ABox->Press_Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_End_Color, ABox->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    }
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short cbutton_idx;
  TCButton *local_cbutton;
  unsigned short box_idx;
  TBox *local_box;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  cbutton_idx = 0;
  box_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    save_bled_direction = TFT_BLED_Direction;
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Set_Active(Set_Index, Write_Command, Write_Data);
    TFT_Init(CurrentScreen->Width, CurrentScreen->Height);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 13, 12);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
    TFT_BLED_Direction = save_bled_direction;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (cbutton_idx < CurrentScreen->CButtonsCount) {
      local_cbutton = GetCButton(cbutton_idx);
      if (order == local_cbutton->Order) {
        cbutton_idx++;
        order++;
        DrawCButton(local_cbutton);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  cbutton_order       = -1;
  box_order           = -1;
  //  CButtons
  for ( _object_count = 0 ; _object_count < CurrentScreen->CButtonsCount ; _object_count++ ) {
    local_cbutton = GetCButton(_object_count);
    if (local_cbutton->Active == 1) {
      if (IsInsideObject(X, Y, local_cbutton->Left, local_cbutton->Top,
                         local_cbutton->Width, local_cbutton->Height) == 1) {
        cbutton_order = local_cbutton->Order;
        exec_cbutton = local_cbutton;
      }
    }
  }

  //  Boxes
  for ( _object_count = 0 ; _object_count < CurrentScreen->BoxesCount ; _object_count++ ) {
    local_box = GetBox(_object_count);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  _object_count = -1;
  if (cbutton_order >  _object_count )
    _object_count = cbutton_order;
  if (box_order >  _object_count )
    _object_count = box_order;
}


static void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_cbutton        = 0;
  exec_box            = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == cbutton_order) {
      if (exec_cbutton->Active == 1) {
        if (exec_cbutton->OnPressPtr != 0) {
          exec_cbutton->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // CButton
    case 8: {
      if (PressedObject != 0) {
        exec_cbutton = (TCButton*)PressedObject;
        if ((exec_cbutton->PressColEnabled == 1) && (exec_cbutton->OwnerScreen == CurrentScreen)) {
          DrawCButton(exec_cbutton);
        }
        break;
      }
      break;
    }
    // Box
    case 6: {
      if (PressedObject != 0) {
        exec_box = (TBox*)PressedObject;
        if ((exec_box->PressColEnabled == 1) && (exec_box->OwnerScreen == CurrentScreen)) {
          DrawBox(exec_box);
        }
        break;
      }
      break;
    }
  }


  Get_Object(X, Y);


  if (_object_count != -1) {
  // CButtons
    if (_object_count == cbutton_order) {
      if (exec_cbutton->Active == 1) {
        if (exec_cbutton->OnUpPtr != 0)
          exec_cbutton->OnUpPtr();
        if (PressedObject == (TPointer)exec_cbutton)
          if (exec_cbutton->OnClickPtr != 0)
            exec_cbutton->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes
    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (TPointer)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_cbutton        = 0;
  exec_box            = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == cbutton_order) {
      if (exec_cbutton->Active == 1) {
        if (exec_cbutton->PressColEnabled == 1) {
          object_pressed = 1;
          DrawCButton(exec_cbutton);
        }
        PressedObject = (TPointer)exec_cbutton;
        PressedObjectType = 8;
        if (exec_cbutton->OnDownPtr != 0) {
          exec_cbutton->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->PressColEnabled == 1) {
          object_pressed = 1;
          DrawBox(exec_box);
        }
        PressedObject = (TPointer)exec_box;
        PressedObjectType = 6;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
  else {
      USB_Report.y_value = 0;
      USB_Report.x_value = 0;
  }
}

void Init_MCU() {
  PMMODE = 0;
  PMAEN  = 0;
  PMCON  = 0;  // WRSP: Write Strobe Polarity bit
  PMMODEbits.MODE = 2;     // Master 2
  PMMODEbits.WAITB = 0;
  PMMODEbits.WAITM = 1;
  PMMODEbits.WAITE = 0;
  PMMODEbits.MODE16 = 1;   // 16 bit mode
  PMCONbits.CSF = 0;
  PMCONbits.PTRDEN = 1;
  PMCONbits.PTWREN = 1;
  PMCONbits.PMPEN = 1;
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(76, 907, 77, 915);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}